/**
 * Copyright (C) 2001-2002
 * - France Telecom R&D
 * - Laboratoire Logiciels, Systemes, Reseaux - UMR 5526, CNRS-INPG-UJF
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Release: 1.0
 *
 * Authors:
 *
 */

package org.objectweb.perseus.pool.api;

import org.objectweb.perseus.dependency.api.DeadLockException;

import java.util.Collection;


/**
 * The interface <b>Pool</b> defines an object that pools resources of any
 * kind. Resources must be requested (getResource) and released
 * (releaseResource) on demand. A Pool object can be parameterized along
 * different dimensions. All these dimensions are represented by PoolAttributes
 * interface.
 */
public interface Pool {

	/**
	 * <b>getResource</b> is used to allocate a PoolResource from the Pool.
	 * Some hints are passed in order to specialise the matching or creation
	 * of PoolResource.
	 * @param hints	Some properties to specialise the matching or the creation
	 *				of PoolResource.
	 * @return		The PoolResource allocated from the Pool.
	 */
	Object getResource(Object hints) throws PoolException;

	/**
	 * <b>getResource</b> is used to allocate a PoolResource from the Pool.
	 * Some hints are passed in order to specialise the matching or creation
	 * of PoolResource. The user parameter avoids dead lock by the use of a
	 * dependency graph.
	 * @param hints	Some properties to specialise the matching or the creation
	 *				of PoolResource.
	 * @param user is an identifier of the context wanting a resource in the
	 * pool.
	 * @return		The PoolResource allocated from the Pool.
	 */
	Object getResource(Object hints, Object user)
			throws PoolException, DeadLockException;

	/**
	 * <b>releaseResource</b> releases a PoolResource in order to allow the
	 * Pool to recycle this PoolResource.
	 * @param resource	The PoolResource to be released.
	 */
	void releaseResource(Object resource) throws PoolException;
	
	/**
	 * @return the number resource used and free
	 */
	int getSize();
	
	/**
	 * @return the number of unsused resource 
	 */
	int getFreeResourceNumber();

	/**
	 * @return the number of used resource 
	 */
	int getUsedResourceNumber();
	
	/**
	 * @return collection of user. The collection can be empty even if resource
	 * are used, in case of the user did not specify any reference on 
	 * getResource. 
	 */
	Collection getUsers();


}
